#!/usr/bin/perl
#

#
#
# Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:#
#
#- Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the following disclaimer.
#
#- Redistributions in binary form must reproduce the above copyright
#  notice, this list of conditions and the following disclaimer in the
#  documentation and/or other materials provided with the distribution.
#
#- Neither the name of Embedded Adventures nor the names of its contributors
#  may be used to endorse or promote products derived from this software
#  without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
# THE POSSIBILITY OF SUCH DAMAGE.
#
# Contact us at admin@embeddedadventures.com


#
#
#
# Run this: 
# calc.pl > chipdefs.h
# 
#
# to regenerate chipdefs.h with new information.
#
# 5 May 2008
# Added 18f2520 thanks to Peter Lawson

print "#ifndef __chipdefs_h\n";
print "#define __chipdefs_h\n";

             #chip,         family, memory, bootloader_size, min_erase_chunk, min_write_chunk, max_write_chunk, avoid_top_of_mem, tris setting 
print_config("PIC16F88",    "pic_16", 4096,  320, 32, 4, 4,  0,"trisb = 0b00100100;  // trisb 5,2 = 1");
print_config("PIC16F876A",  "pic_16", 8192,  249, 0,  4, 4,  0,"trisc = 0b11000000;  // trisc 7,6 = 1");
print_config("PIC16F877A",  "pic_16", 8192,  249, 0,  4, 4,  0,"trisc = 0b11000000;  // trisc 7,6 = 1");
# below was 520
print_config("PIC18F2620",  "pic_18", 65536, 600, 64, 1, 64, 0,"trisc = 0b11000000;  // trisc 7,6 = 1");
print_config("PIC18F4520",  "pic_18", 32768, 520, 64, X, 32, 0,"trisc = 0b11000000;  // trisc 7,6 = 1");
print_config("PIC18F4550",  "pic_18", 32768, 520, 64, X, 32, 0,"trisc = 0b11000000;  // trisc 7,6 = 1");
print_config("PIC18F2520",  "pic_18", 32768, 520, 64, X, 32, 0,"trisc = 0b11000000;  // trisc 7,6 = 1");
print_config("PIC18F452",   "pic_18", 32768, 534, 64, X, 8,  0,"trisc = 0b11000000;  // trisc 7,6 = 1");
print_config("PIC18F26K20", "pic_18", 65536, 600, 64, 1, 64, 0,"trisc = 0b11000000;  // trisc 7,6 = 1");
print_config("PIC18F25K20", "pic_18", 32768, 600, 64, 1, 64, 0,"trisc = 0b11000000;  // trisc 7,6 = 1");
print_config("PIC18F14K50", "pic_18", 16384, 600, 64, 16, 16, 0,"trisb.5 = 1; anselh.3 = 0; // trisb 5 = 1, AN11 turned off");

#// 128k less 8 words (16 bytes) for stored config
print_config("PIC18F67J50","pic_18", 131072,536, 1024, X, 64,  8, "trisc = 0b10000000;  // trisc 7 = 1, 6=0");

print "#endif\n";

sub print_config {

my ($chip, $family, $memory, $bl_size, $min_erase_chunk, $min_write_chunk, $max_write_chunk, $avoid_tom, $serial_tris_setup) = @_;
my ($mem_div);


if ($family == "pic_18") {
	$mem_div = 2;
} else {
	$mem_div = 1;
}	

# Allow 3 instructions grace, boostloader gets a bit confused otherwise
my ($bloader_start) = $memory - 3 - $bl_size - $avoid_tom;
if ($mem_div == 2) {
	$bloader_start = $bloader_start >> 1;
	$bloader_start = $bloader_start << 1; # Chop off this bit
}	

print "#ifdef _$chip\n";
print "    #warning \"Compiling for $chip\"\n";
print "    #define pic_mem_". $memory / 1024 . "k\n";
print "    #define $family\n";
if ($min_erase_chunk > 0) {
	print "    #define erase_before_write\n";
	print "    #define min_erase_chunk $min_erase_chunk\n";
}
printf "    #define max_write_chunk	$max_write_chunk\n";
printf "    #define BLOADER_START 0x%.4x\n", $bloader_start;
printf "    #define BLOADER_UPPER 0x%.2x\n", $bloader_start >> 16;
printf "    #define BLOADER_START_HIGH 0x%.2x\n", ($bloader_start >> 8) & 0xff; 
printf "    #define BLOADER_START_LOW  0x%.2x\n", $bloader_start & 0xff;
printf "    #warning \"Did you set -rb ".$bloader_start." in Settings | Options | Linker?\"\n";
if ($family eq "pic_16") {
	if ($min_erase_chunk == 32){
		$moved_bvector_start = ($bloader_start & 0xffe0) - 8;
	} elsif ($min_erase_chunk == 0) {	
		$moved_bvector_start = $bloader_start - 8;
	} else {
		printf "!!- Well, don't know what to do with this erase chunk (%d) for moved_bvector_start\n", $min_erase_chunk;
	}
} else {
	
	#// for 18f2620, 18f4520 which has to erase on 64 byte (32 word) boundaries:
	#// MOVED_BVECTOR_START  = (BLOADER_START & 0xFFC0) - 8

	if ($min_erase_chunk > 0) { #// was 0xffc0
		$moved_bvector_start = ($bloader_start & (0xfffff - ($min_erase_chunk -1))) - 8;
	} 
	#//else {
	#//	printf "!!- Well, don't know what to do with this erase chunk (%d) for moved_bvector_start\n", $min_erase_chunk;
	#//}
}

printf "    #define MOVED_BVECTOR_START 0x%.4x\n", $moved_bvector_start;
printf "    #define MOVED_BVECTOR_UPPER 0x%.2x\n", $moved_bvector_start >> 16;
printf "    #define MOVED_BVECTOR_HIGH  0x%.2x\n", ($moved_bvector_start >> 8) & 0xff;
printf "    #define MOVED_BVECTOR_LOW   0x%.2x\n", $moved_bvector_start & 0xff;

my ($jump_to_bvector, $jump_to_bvector1, $jump_to_bvector2);

if ($family eq "pic_16") {
	#// To Calculate: 0x2800 + (MOVED_BVECTOR_START & 0x7FF)
	$jump_to_bvector = 0x2800 + ($moved_bvector_start & 0x7ff);
	
	printf "    #define JUMP_TO_BVECTOR 0x%.4x\n", $jump_to_bvector;
	
} else {
	
	#// 18f series pics
	#// Require two instructions for jump
	
	#// To Calculate: 
	#// EF00 + ((MOVED_BVECTOR_START >> 1) & 0x00ff)
	#// F000 + ((MOVED_BVECTOR_START >> 1) & 0x7ff00) >> 8)


	$jump_to_bvector1 = 0xef00 + (($moved_bvector_start >> 1) & 0x00ff);
	$jump_to_bvector2 = 0xf000 + (($moved_bvector_start >> 1) >> 8);

	printf "    #define JUMP_TO_BVECTOR1 0x%.4x\n", $jump_to_bvector1;
	printf "    #define JUMP_TO_BVECTOR2 0x%.4x\n", $jump_to_bvector2;

}


if ($family eq "pic_16") {

	#//--#pragma cdata[0] = /*BSF*/0x1400 + /*PCLATH*/10 + (/*bit*/3 << 7)
	#//--#pragma cdata[1] = /*BSF*/0x1400 + /*PCLATH*/10 + (/*bit*/4 << 7)
	#//--#pragma cdata[2] = /*GOTO*/0x2800 + (BLOADER_START & 0x7FF)

	printf "    #pragma DATA 0x000, 0x158A // BSF PCLATH, 3\n";
	printf "    #pragma DATA 0x001, 0x160A // BSF PCLATH, 4\n";
	printf "    #pragma DATA 0x002, 0x%.4x // Goto bloaderstart\n", 0x2800 + ($bloader_start & 0x7ff);

} else { # pic_18

	#// To Calculate: 
	#// EF00 + ((BLOADER_START >> 1) & 0x00ff)
	#// F000 + ((BLOADER_START >> 1)) >> 8
	#// Goto 0x7ee0
	#// 7ee0 >> 1 = 3f70
	#// Seem to need to put these backwards
	

	printf "    #pragma DATA 0x0000,  0x%.2x, 0xEF, 0x%.2x, 0xf0\n", ($bloader_start >> 1) & 0xff, ($bloader_start >> 1) >> 8;
	printf "    #define JUMP_TO_BLOADER1 0xef\n";
	printf "    #define JUMP_TO_BLOADER2 0x%.2x\n", (($bloader_start >> 1) & 0x00ff);
	printf "    #define JUMP_TO_BLOADER3 0xf0\n";
	printf "    #define JUMP_TO_BLOADER4 0x%.2x\n", (($bloader_start >> 1) >> 8);
	
}


printf "    #define SERIAL_TRIS_SETUP() $serial_tris_setup\n";

print "#endif\n";
print "\n";
print "\n";
print "\n";

}