//-----------------------------------------------------
// Pic Pack library
// 
// draw_ea_ldp6416.c
//
// Demonstration routines showing the draw library
// used with the Embedded Adventures LDP6416 board
//
// (c) Embedded Adventures 2010
// iharris [at] embeddedadventures.com
//
//
//-----------------------------------------------------

// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "draw\draw.h"
// System includes
#include "string.h"
#include "stdlib.h"
#include "ea_ldp6416.h"
#include "draw\draw_screen_buffer.h"
#include "m41t81s.h"
#include "tmp75.h"
// Support for Timer 0
#include "pic_timer.h"
// Support for Timer 1
#include "pic_timer1.h"
#include "pic_term.h"
#include "draw\draw_tests.h"
#include "pic_tick.h"

uns8 minutes;
uns8 hours;
uns8 seconds;
uns8 day;
uns8 date;

uns8 month;
uns8 year;

bit seconds_trigger = 0;

void update_clock() {

	
	// grab the latest time from the RTC chip

	minutes = rtc_get_minutes();
	hours   = rtc_get_hours();
	seconds = rtc_get_seconds();
	day 	= rtc_get_dow();
	date 	= rtc_get_date();
	month 	= rtc_get_month();
	year 	= rtc_get_year();
}

// Interrupt routine - - - - - - - - - -
void interrupt() {
	
	// very important for timer 1 handler to be first: It refreshes
	// the display 
	timer_handle_1_isr();

	serial_handle_tx_isr();
	serial_handle_rx_isr();

	// Handle trigger to update clock every second
	if (test_bit(intcon3, INT1IF)) {
		clear_bit(intcon3, INT1IF);
		seconds_trigger = 1;
	}
	// Handle timer 0 interrupt
	timer_handle_0_isr();	
	
}

void timer_1_callback() {
	// refresh display from draw buffer
	drv_refresh();	
}

// Timer 0 callback handled by pic_tick

void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
	
	first_char = term_buffer[0];
	switch (first_char) {
	case '1':
		drv_print_buffer();
		break;
	case '2':
		
		break;
	case '3':
		
		break;
	case 'a':
		
		break;
	case 'b':
		break;
		
	case 'c':
		break;
	case 'd':
		break;
	case 'u':
		break;
	case 'p':
		break;
		
	} // switch		

}

	
// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs

	// temp sensor setup ----------------------------

	tmp75_setup_io();
	
	// 12 bit temp, continue temp, shutdown=0
	// OS R1 R0 F1 F0 POL TM SD
	// 0  1  1  0  0  0   0  0   
	tmp75_set_config(TMP75_ADDR, 0b01100000);

	// real time clock setup ----------------------------

	rtc_setup_io();

	rtc_start_clock();	// unpause the m41t81s
	rtc_set_sqw_freq(rtc_sqw_freq_1Hz);	// give us a shove every second
	rtc_start_sqw_output();	// turn on the 1Hz output
	
	make_input(PORTB, 1);	// sqw wired to portb.1
	clear_bit(intcon3, INT1IF);
	set_bit(intcon3, INT1IE);		// enable int1
	
	
	serial_setup(SPBRG_115200);

	term_init();	// initialise terminal 

	// LED setup ----------------------------
	
	make_output(PORTC, 1);	// portc.1 is our led 
	clear_pin(PORTC, 1);	// make it an output, turn it off

	// general setup ----------------------------

	// Timer 0 is used for ticks - 1ms at 40mhz (10mips)
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 10000 -1); 

	// Timer 1 is used for led board refresh
	timer_setup_1(TIMER1_PRESCALER_OFF, 0xffff - 2500);

	turn_peripheral_ints_on();
	turn_global_ints_on();

	draw_setup_io();
	draw_init();

}




void main() {


uns8 test_result, test_count;
uns16 marker_tick, test_tick;

	configure_system();
	delay_ms(100);

	serial_print_str("\n\nPicPack EA LDP-6416 Led Display demo\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("total buffer = ");
	serial_print_int(DRAW_TOTAL_BUFFER_SIZE);
	serial_print_nl();
	serial_print_str("wide = ");
	serial_print_int(DRAW_PIXELS_WIDE);
	serial_print_nl();
	serial_print_str("high = ");
	serial_print_int(DRAW_PIXELS_HIGH);
	serial_print_nl();
	serial_print_str("ppb = ");
	serial_print_int(DRAW_PIXELS_PER_BYTE);
	serial_print_nl();
	
	timer_start_0();
	timer_start_1();
	
	test_count = 0;
	marker_tick = tick_get_count();	// find out what we're up to

	for(;;) {	// for ever
		
		term_process();	// process any typing on the terminal
			
		if (seconds_trigger) {
			update_clock();
			seconds_trigger = 0;
		}
		
		test_tick = tick_get_count();	// find out what we're up to

		if (tick_calc_diff(marker_tick, test_tick) >= 2000) {	// 1000 - it's a second, 2000 = 2 seconds
			serial_print_str("Test\n");
			delay_ms(50);
			test_result = draw_tests_run(test_count);
			if (test_result == TEST_RESULT_NO_MORE_TESTS) {
				test_count = 255;
			}
			while (test_result == TEST_RESULT_NOT_APPLICABLE) {	// No test
				test_count++;	// increment test
				test_result = draw_tests_run(test_count);
			}
			test_count++;
			marker_tick = test_tick;
			serial_print_str("Done\n");
		}
		


		

	}
	
}	// main

