//-----------------------------------------------------
// Pic Pack library
// 
// ea_dsP7s04_demo.c
//
// Demonstration routines for Embedded Adventures 
// 7 segment x4 display module
//
// Ian Harris 2010
// www.embeddedadventures.com
//
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "cat4016.h"
#include "ea_dsp7s04.h"

// System includes
#include "string.h"
#include "stdlib.h"
#include "config.h"
#include "pic_term.h"



// Dummy variable to show we can set it 

uns8 my_var = 0;
uns8 my_var2 = 0;
// Interrupt routine - - - - - - - - - -

void interrupt() {
	
		serial_handle_tx_isr();
		serial_handle_rx_isr();
}


//
// term_entry_callback will be called when 
// user pressed <enter> on serial "terminal".
// term_buffer contains the characters (if any)
// that the user has entered.

void term_entry_callback(uns8 *term_buffer) {

uns8 count;
uns8 display[5];

	if (strlen(term_buffer) < 4) {
		serial_print_str("Not enough characters! Type 4 characters and press enter\n");
	} else {	
		strncpy(display, term_buffer, 4);
		display[4] = '\0';
		ea_dsp7s04_print_str(display);
	}

}	// term_entry_callback


// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	
	
	// turn off interrupts just in case
	kill_interrupts();
	
	// kill those pesky analogue inputs
	turn_analog_inputs_off();	
	
	// set serial to run at 115200bps 
	serial_setup(SPBRG_115200);
	
	// initialise termainl
	term_init();
	
	ea_dsp7s04_setup_io();

	// turn peripheral interrupts on (ie, serial port)
	turn_peripheral_ints_on();
	
	// turn all interrupts on
	turn_global_ints_on();
}



void main() {
	
	configure_system();
	
	delay_ms(100);	// mostly so screamer bootloader has time to switch windows
	
	serial_print_str("\n\nDSP-7S04 demo\n");
	serial_print_str( "\n<");
	
	// Print the time and date this file was compiled. Handy to know which
	// version you are running.
	
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Type 4 characters and press enter\n");
	ea_dsp7s04_print_str("HELO");
	

	for(;;) {

		term_process();
	
	}

	
}	// main

