/*

Demo of RTC / Temp Sensor module

*/

// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "pic_term.h"
#include "m41t81s.h"
#include "tmp75.h"
#include "convert.h"

// System includes
#include "string.h"
#include "stdlib.h"


// Interrupt routine - - - - - - - - - -
void interrupt() {
	
	serial_handle_rx_isr();
	serial_handle_tx_isr();
}


void request_time() {

uns8 minutes, hours, seconds, day, date, month, year;
char buffer[10];

	minutes = rtc_get_minutes();
	hours   = rtc_get_hours();
	seconds = rtc_get_seconds();
	day 	= rtc_get_dow();
	date 	= rtc_get_date();
	month 	= rtc_get_month();
	year 	= rtc_get_year();
	
	if (hours > 9) {
		buffer[0] = '0' + hours / 10;
		buffer[1] = '0' + hours % 10;
	} else {
		buffer[0] = ' ';
		buffer[1] = '0' + hours;
	}
	buffer[2] = ':';
	
	buffer[3] = '0' + minutes / 10;
	buffer[4] = '0' + minutes % 10;
	
	buffer[5] = ':';

	buffer[6] = '0' + seconds / 10;
	buffer[7] = '0' + seconds % 10;
	buffer[8] = 0;

	serial_print_str("The time is ");
	serial_print_str(buffer);
	
	serial_print_nl();
	
	serial_print_str("The day is ");
	
	switch (day) {
		case 1 : serial_print_str("Mon"); break;
		case 2 : serial_print_str("Tues"); break;
		case 3 : serial_print_str("Wednes"); break;
		case 4 : serial_print_str("Thurs"); break;
		case 5 : serial_print_str("Fri"); break;
		case 6 : serial_print_str("Satur"); break;
		case 7 : serial_print_str("Sun"); break;
	}
	serial_print_str("day\n");
	
	serial_print_str("The date is ");
	
	serial_print_int(date);
	serial_print_str(" of ");
	
	switch (month) {
		case 1 : serial_print_str("Jan"); break;
		case 2 : serial_print_str("Feb"); break;
		case 3 : serial_print_str("Mar"); break;
		case 4 : serial_print_str("Apr"); break;
		case 5 : serial_print_str("May"); break;
		case 6 : serial_print_str("Jun"); break;
		case 7 : serial_print_str("Jul"); break;
		case 8 : serial_print_str("Aug"); break;
		case 9 : serial_print_str("Sep"); break;
		case 10: serial_print_str("Oct"); break;
		case 11: serial_print_str("Nov"); break;
		case 12: serial_print_str("Dec"); break;
	}
	
	serial_print_str(" 20");
	if (year < 10) {
		serial_putc('0');
	}	
	serial_print_int(year);	
		
}

void request_temp() {
	
char buffer[8];
uns8 count;
uns8 temp_int, temp_dec;
bit negative = 0;
uns16 temp;

	temp = tmp75_get_temp(TMP75_ADDR);

	if (temp.15 == 1) {	// negative
		negative = 1;
		temp = temp | 0x0f;	// Set lower 4 bits
		temp = ~temp;	// Take complement
		temp = temp + 1;		// Add one
	}

	temp_int = temp >> 8;
	temp_dec = (temp & 0xff) >> 5;
	temp_dec = convert_to_dec2(temp_dec);
	count = 0;

	if (negative) {
		buffer[0] = '-';
		count = 1;
	}
	
	if (temp_int > 9) {
		buffer[count++] = '0' + temp_int / 10;
		buffer[count++] = '0' + temp_int % 10;
	} else {
		buffer[count++] = '0' + temp_int;
	}
	
	buffer[count++] = '.';
	
	if (temp_dec>9) {
		buffer[count++] = '0' + temp_dec / 10;
		buffer[count++] = '0' + temp_dec % 10;
	} else {
		buffer[count++] = '0';
		buffer[count++] = '0' + temp_dec;
	}
	buffer[count++] = '\0';
	serial_print_str(buffer);
}

void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
uns8 setting;

	first_char = term_buffer[0];

	if ((first_char == 's') && (strlen(term_buffer)>2)) {
		uns8 setting = atoui_dec(&term_buffer[2]);
		serial_print_str(" Setting to ");
		serial_print_int(setting);
		serial_putc(' ');
		switch (term_buffer[1]) {
			case 's' :
				rtc_set_seconds(setting);
				serial_print_str("sec");
				break;
			case 'm' :
				rtc_set_minutes(setting);
				serial_print_str("min");
				break;
			case 'h' :
				rtc_set_hours(setting);
				serial_print_str("hour");
				break;
			case 'a' :
				rtc_set_day(setting);
				serial_print_str("day");
				break;
			case 't' :
				rtc_set_date(setting);
				serial_print_str("date");
				break;
			case 'o' :
				rtc_set_month(setting);
				serial_print_str("month");
				break;
			case 'y' :
				rtc_set_year(setting);
				serial_print_str("year");
				break;
			default:
				serial_print_str(" huh?");		
		}
		serial_putc('\n');	
	} else {
		switch (first_char) {
			case 't':
				serial_print_str("Requesting time:\n");
				request_time();
				serial_print_nl();
				break;
			case 'm':
				request_temp();

				serial_print_nl();
				break;
			default:
				serial_print_str("Unknown command\n");
		}	// switch	
	} 	

}

 
// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case
	
	turn_analog_inputs_off();	// kill those pesky analogue inputs
	
	
	serial_setup(SPBRG_115200);

	// temp sensor setup ----------------------------

	tmp75_setup_io();
	
	// 12 bit temp, continue temp, shutdown=0
	// OS R1 R0 F1 F0 POL TM SD
	// 0  1  1  0  0  0   0  0   
	tmp75_set_config(TMP75_ADDR, 0b01100000);

	// real time clock setup ----------------------------
	
	rtc_setup_io();

	rtc_start_clock();	// unpause the m41t81s
	rtc_set_sqw_freq(rtc_sqw_freq_1Hz);	// give us a shove every second
	rtc_start_sqw_output();	// turn on the 1Hz output

	// setup terminal ----------------------------

	term_init();	// initialise terminal 

	// turn on interrupts ----------------------------
	
	turn_peripheral_ints_on();
	turn_global_ints_on();
}

	


void main() {
	configure_system();

	delay_ms(250);
	delay_ms(250);
	delay_ms(250);
	delay_ms(250);
	delay_ms(250);

	serial_print_str("\n\nPIC terminal\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands:\n");
	serial_print_str("t - Request time\n");
	serial_print_str("m - Request temp\n");
	serial_print_str("sX05 - Set X to 05\n");
	serial_print_str("X=\n");
	serial_print_str("s sec\nm min\nh hour\n");
	serial_print_str("a day\nt date\no month\ny year\n");
	

	for (;;) {
		term_process();
	}	// while (1)

	
}	// main

