/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/



#include "audio_queue.h"
#include "somo_14d.h"
#include "pic_serial.h"

/** Audio queue fifo  */
uns8 audio_queue_fifo[AUDIO_QUEUE_FIFO_SIZE];
/** Audio queue fifo start point */
uns8 aq_start = 0;
/** Audio queue fifo end point */
uns8 aq_end = 0;
/** Audio playing at present */
bit audio_playing = 0;

void audio_queue_add(uns8 phrase) {

uns8 aq_next;

    if ((aq_end == aq_start) &&  // Nothing in the fifo
		(audio_playing == 0)) {  // And txreg is empty
				
		somo_14d_set_file_id(phrase);
		audio_playing =  1;
	} else {	// Put it in the buffer
		aq_next = aq_end + 1;	// Get next buffer position
		if (aq_next == AUDIO_QUEUE_FIFO_SIZE) {	// If we're at the end
			aq_next = 0;	// wrap to the beginning
		}
		if (aq_next == aq_start) {
			// we're full!
			return;	// just forget it for now
		}	
			
		audio_queue_fifo[aq_end] = phrase; // put it in
		aq_end = aq_next;  // move pointer along
	}	
}

/** Call when audio file completes.

This routine will pluck the next file off the queue
and start playing it. Assumes it is in an interrupt 
otherwise will need wrapping in critsec */

void audio_queue_process()	
{
uns8 aq_next;

	if (aq_end == aq_start) { // anything in the fifo?
		//serial_print_str("End of queue\n");
		audio_playing = 0;
	   return; // nope
	}
	aq_next = aq_start + 1;	// get next position
	if (aq_next == AUDIO_QUEUE_FIFO_SIZE) {	// if we're at the end of the buffer
		aq_next = 0;	// wrap to the beginning
	}
	if (aq_end == aq_next) {	// if we've only got one character to send
		//?????clear_bit(pie1, TXIE);	// then turn off interrupts
	}
	audio_playing = 1;
				
	somo_14d_set_file_id(audio_queue_fifo[aq_start]);
	aq_start = aq_next;	// move start position of fifo

}


void audio_queue_clear() {
	// crit sec
	start_crit_sec();
	aq_end = aq_start;
	if (audio_playing || somo_14d_is_busy()) {
		serial_print_str("<<BUSY>>");
		delay_ms(10);
		somo_14d_stop();
		while (somo_14d_is_busy()) {};
		delay_ms(10);
	}	
	audio_playing = 0;
	end_crit_sec();
	// crit sec end
}	

uns8 audio_queue_empty() {
    return (audio_playing == 0);
}