/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

#include "ccld.h"

/*

Constant current LED driver IC interface code

ccld
TLC2846
CAT4016
etc

*/


void ccld_latch_data() {

	set_pin(ccld_lat_port, ccld_lat_pin);
	clear_pin(ccld_lat_port, ccld_lat_pin);

}

void ccld_setup_io() {

	clear_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_lat_port, ccld_lat_pin);
#ifdef ccld_blk_port
	set_pin(ccld_blk_port, ccld_blk_pin);
#endif	
	make_output(ccld_sin_port, ccld_sin_pin);
	make_output(ccld_clk_port, ccld_clk_pin);
#ifdef ccld_blk_port
	make_output(ccld_blk_port, ccld_blk_pin);
#endif
	make_output(ccld_lat_port, ccld_lat_pin);
	
}
	

void ccld_write_data(uns16 d) {

	ccld_write_data_byte(d >> 8);
	ccld_write_data_byte(d & 0xff);

}	


void ccld_write_data_byte(uns8 d) {

uns8 count;

	for (count=0; count<8; count++) {
	
		// set data
		change_pin_var(ccld_sin_port, ccld_sin_pin, d.7);
	
		// pulse clk

		set_pin(ccld_clk_port, ccld_clk_pin);
		d <<= 1;


		clear_pin(ccld_clk_port, ccld_clk_pin);

	}
	delay_us(100);
}

void ccld_enable_display(uns8 on){

#ifdef ccld_blk_port

	if (on){
		clear_pin(ccld_blk_port, ccld_blk_pin);
	} else {
		set_pin(ccld_blk_port, ccld_blk_pin);
	}
#endif
}	


// this will only work if you have your blk pin defined!

void ccld_tlc_special_mode(void) {

	#ifdef ccld_blk_port
		set_pin(ccld_blk_port, ccld_blk_pin);
	#endif
	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);

	// set OE low
	#ifdef ccld_blk_port
		clear_pin(ccld_blk_port, ccld_blk_pin);
	#endif
	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);
	
	// set OE high
	#ifdef ccld_blk_port
		set_pin(ccld_blk_port, ccld_blk_pin);
	#endif
	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);

	// now set latch high
	set_pin(ccld_lat_port, ccld_lat_pin);
	
	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);

	// latch low
	clear_pin(ccld_lat_port, ccld_lat_pin);
	
	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);
}
	
// this will only work if you have your blk pin defined!
void ccld_tlc_normal_mode(void) {
	#ifdef ccld_blk_port
		set_pin(ccld_blk_port, ccld_blk_pin);
	#endif
	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);

	// set OE low
	#ifdef ccld_blk_port
		clear_pin(ccld_blk_port, ccld_blk_pin);
	#endif
	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);
	
	// set OE high
	#ifdef ccld_blk_port
		set_pin(ccld_blk_port, ccld_blk_pin);
	#endif
	
	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);

	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);

	// pulse clk
	set_pin(ccld_clk_port, ccld_clk_pin);
	clear_pin(ccld_clk_port, ccld_clk_pin);
	

}
	
