/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com
*/

/** \file draw_tests.c
    \brief Routines to test the Draw graphics functions 
*/

#include "draw_tests.h"
#include "pic_serial.h"
#include "draw\draw.h"
#include "draw\ea_bitmaps.h"
#include "config.h"


uns8 draw_tests_run(uns8 test_num) {

uns8 test_result;
	test_result = TEST_RESULT_RAN;
	draw_clear_screen();

	switch (test_num) {
		case 0:
				serial_print_str("0:Top left pixel\n");
				draw_set_pixel(0, DRAW_TOP_PIXEL_Y, 1);
				break;
				
		case 1:
				serial_print_str("1:Top right pixel\n");
				draw_set_pixel(DRAW_PIXELS_WIDE - 1, DRAW_TOP_PIXEL_Y, 1);
				break;
				
		case 2:
				serial_print_str("2:Bottom right pixel\n");
				draw_set_pixel(DRAW_PIXELS_WIDE - 1, DRAW_BOTTOM_PIXEL_Y, 1);
				break;
				
		case 3:
				serial_print_str("3:Bottom left pixel\n");
				draw_set_pixel(0, DRAW_BOTTOM_PIXEL_Y, 1);
				break;
				
		case 4:
				serial_print_str("4:Full screen colour 1\n");
				draw_rect(0, DRAW_TOP_PIXEL_Y, DRAW_PIXELS_WIDE, DRAW_PIXELS_HIGH, 1);
				break;
				
		case 5:
				#if DRAW_BITS_PER_PIXEL > 1
					serial_print_str("5:Full screen colour 2\n");
					draw_rect(0, DRAW_TOP_PIXEL_Y, DRAW_PIXELS_WIDE, DRAW_PIXELS_HIGH, 2);
				#else
					serial_print_str("5:Not applicable\n");
					test_result = TEST_RESULT_NOT_APPLICABLE;
				#endif	
				break;
				
		case 6:
				#if DRAW_BITS_PER_PIXEL > 1
					serial_print_str("6:Full screen colour 3\n");
					draw_rect(0, DRAW_TOP_PIXEL_Y, DRAW_PIXELS_WIDE, DRAW_PIXELS_HIGH, 3);
						#else
					serial_print_str("6:Not applicable\n");
					test_result = TEST_RESULT_NOT_APPLICABLE;
				#endif	
				break;
				
		case 7:
				serial_print_str("7:Circle\n");
				// circle 2 is centred mid-pixel, good for even number of pixels
				#if DRAW_PIXELS_WIDE > DRAW_PIXELS_HIGH
					#define TEST_RADIUS DRAW_PIXELS_HIGH / 2 -1
				#else
					#define TEST_RADIUS DRAW_PIXELS_WIDE / 2 -1
				#endif
				draw_circle2(DRAW_PIXELS_WIDE / 2 -1, DRAW_PIXELS_HIGH / 2 -1, TEST_RADIUS, 1);
				break;
				
		case 8:
				serial_print_str("8:Rect 1 pixel from edges colour 1\n");
				#if DRAW_TOP_PIXEL_Y == 0
					draw_rect(1, DRAW_TOP_PIXEL_Y + 1, DRAW_PIXELS_WIDE -2, DRAW_PIXELS_HIGH -2, 1);
				#else
					draw_rect(1, DRAW_TOP_PIXEL_Y - 1, DRAW_PIXELS_WIDE -2, DRAW_PIXELS_HIGH -2, 1);
				#endif
				break;
				
		case 9:
				#if DRAW_BITS_PER_PIXEL > 1
					serial_print_str("9:Rect 1 pixel from edges colour 2\n");
					#if DRAW_TOP_PIXEL_Y == 0
						draw_rect(1, DRAW_TOP_PIXEL_Y + 1, DRAW_PIXELS_WIDE -2, DRAW_PIXELS_HIGH -2, 2);
					#else
						draw_rect(1, DRAW_TOP_PIXEL_Y - 1, DRAW_PIXELS_WIDE -2, DRAW_PIXELS_HIGH -2, 2);
					#endif
				#else
					serial_print_str("9:Not applicable\n");
					test_result = TEST_RESULT_NOT_APPLICABLE;
				#endif			
				break;
				
		case 10:
				#if DRAW_BITS_PER_PIXEL > 1
					serial_print_str("10:Rect 1 pixel from edges colour 3\n");
					#if DRAW_TOP_PIXEL_Y == 0
						draw_rect(1, DRAW_TOP_PIXEL_Y + 1, DRAW_PIXELS_WIDE -2, DRAW_PIXELS_HIGH -2, 3);
					#else
						draw_rect(1, DRAW_TOP_PIXEL_Y - 1, DRAW_PIXELS_WIDE -2, DRAW_PIXELS_HIGH -2, 3);
					#endif
				#else
					serial_print_str("10:Not applicable\n");
					test_result = TEST_RESULT_NOT_APPLICABLE;
				#endif		
				break;
				
		case 11:
				serial_print_str("11:Lines around edges\n");
				draw_line(0, DRAW_TOP_PIXEL_Y, 0, DRAW_BOTTOM_PIXEL_Y, 3);	// Left side
				draw_line(DRAW_PIXELS_WIDE - 1, DRAW_TOP_PIXEL_Y, DRAW_PIXELS_WIDE - 1, DRAW_BOTTOM_PIXEL_Y, 3);	// right side
				draw_line(0, DRAW_TOP_PIXEL_Y, DRAW_PIXELS_WIDE - 1, DRAW_TOP_PIXEL_Y, 3);	// top
				draw_line(0, DRAW_BOTTOM_PIXEL_Y, DRAW_PIXELS_WIDE - 1, DRAW_BOTTOM_PIXEL_Y, 3);	// top
				break;
		case 12:
				serial_print_str("12:Static Text\n");
				draw_print_str(0, DRAW_BOTTOM_PIXEL_Y, DRAW_PIXELS_WIDE, 0, 1, "EmbeddedAdventures");
				break;
		case 13:
				serial_print_str("13: 8 pixel high bitmaps\n");
				draw_bitmap(0, DRAW_TOP_PIXEL_Y, 1, &embedded_bitmap);
				#if DRAW_PIXELS_HIGH > 15
					#if DRAW_TOP_PIXEL_Y == 0
						draw_bitmap(0, DRAW_TOP_PIXEL_Y + 8, 2, &adventures_bitmap);
					#else
						draw_bitmap(0, DRAW_TOP_PIXEL_Y - 8, 2, &adventures_bitmap);
					#endif
				#endif	
				break;
				
		case 14: 
				#if DRAW_PIXELS_HIGH > 15
					serial_print_str("14: 16 pixel high bitmaps\n");
					draw_bitmap(DRAW_PIXELS_WIDE / 2 - 17 , DRAW_TOP_PIXEL_Y, 1, &e_bitmap);
					draw_bitmap(DRAW_PIXELS_WIDE / 2, DRAW_TOP_PIXEL_Y, 2, &a_bitmap);
				#else
					serial_print_str("14:Not applicable\n");
				#endif	
				break;
		case 15: 
				#if DRAW_PIXELS_HIGH > 31
					serial_print_str("15: 32 pixel high bitmaps\n");
					draw_bitmap(0 , DRAW_TOP_PIXEL_Y, 1, &e_big_bitmap);
					draw_bitmap(29, DRAW_TOP_PIXEL_Y, 2, &a_big_bitmap);
				#else
					serial_print_str("15:Not applicable\n");
				#endif	
				break;
				
		default:
				test_result = TEST_RESULT_NO_MORE_TESTS;
				
	}
	draw_paint();
	return test_result;
}


