/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com
*/

/** \file drv_sure2416.c
    \brief Draw drivers for HT1632 based display, eg, Sure 2416 display


*/
#ifndef __draw_drv_sure_2416_h
#define __draw_drv_sure_2416_h

#include "config.h"
#include "ht1632.h"
#include "draw\draw.h"
#include "draw\draw_screen_buffer.h"

#if ht1632_displays > 1

void drv_paint_display(uns16 buffer_loc) {

uns8 count;
uns8 x, y, inv_y;
uns8 byte_loc, bit_loc;
uns8 data;

	// send WR command

	// send 1
	set_pin  (ht1632_data_port, ht1632_data_pin);
	// pulse wr
	clear_pin(ht1632_wr_port, ht1632_wr_pin);
	set_pin  (ht1632_wr_port, ht1632_wr_pin);
	
	// send 0
	clear_pin  (ht1632_data_port, ht1632_data_pin);
	// pulse wr
	clear_pin(ht1632_wr_port, ht1632_wr_pin);
	set_pin  (ht1632_wr_port, ht1632_wr_pin);

	// send 1
	set_pin  (ht1632_data_port, ht1632_data_pin);
	// pulse wr
	clear_pin(ht1632_wr_port, ht1632_wr_pin);
	set_pin  (ht1632_wr_port, ht1632_wr_pin);

	// send mem address of zero
	clear_pin(ht1632_data_port, ht1632_data_pin);
	
	// write mem addr, bits 6 -> 0
	for(count = 0 ; count < 7 ; count++) {
		// pulse wr
		clear_pin(ht1632_wr_port, ht1632_wr_pin);
		set_pin  (ht1632_wr_port, ht1632_wr_pin);
	}

	for (count = 0; count < 48; count++) {
		data = draw_buffer0[buffer_loc];
		for (bit_loc = 0; bit_loc < 8; bit_loc++) {
			if (data.0) {
				set_pin(ht1632_data_port, ht1632_data_pin);
			} else {
				clear_pin(ht1632_data_port, ht1632_data_pin);
			}
			data = data >> 1;	
			clear_pin(ht1632_wr_port, ht1632_wr_pin);
			set_pin  (ht1632_wr_port, ht1632_wr_pin);
		}
		buffer_loc++;	
	}	
	
}	

void drv_paint() {

uns16 buffer_loc;

	buffer_loc = 0;
		
	#if ht1632_displays > 0	
		clear_pin(ht1632_cs1_port, ht1632_cs1_pin);
		drv_paint_display(buffer_loc);
		set_pin(ht1632_cs1_port, ht1632_cs1_pin);
	#endif
	#if ht1632_displays > 1
		buffer_loc = buffer_loc + 48;
		clear_pin(ht1632_cs2_port, ht1632_cs2_pin);
		drv_paint_display(buffer_loc);
		set_pin(ht1632_cs2_port, ht1632_cs2_pin);
	#endif
	#if ht1632_displays > 2
		buffer_loc = buffer_loc + 48;
		clear_pin(ht1632_cs3_port, ht1632_cs3_pin);
		drv_paint_display(buffer_loc);
		set_pin(ht1632_cs3_port, ht1632_cs3_pin);
	#endif
	#if ht1632_displays > 3
		buffer_loc = buffer_loc + 48;
		clear_pin(ht1632_cs4_port, ht1632_cs4_pin);
		drv_paint_display(buffer_loc);
		set_pin(ht1632_cs4_port, ht1632_cs4_pin);
	#endif
}

#else

void drv_paint() {

uns8 count;
uns8 x, y, inv_y;
uns16 buffer_loc;
uns8 byte_loc, bit_loc;
uns8 data;

	clear_pin(ht1632_cs1_port, ht1632_cs1_pin);
	
	// send WR command

	// send 1
	set_pin  (ht1632_data_port, ht1632_data_pin);
	// pulse wr
	clear_pin(ht1632_wr_port, ht1632_wr_pin);
	set_pin  (ht1632_wr_port, ht1632_wr_pin);
	
	// send 0
	clear_pin  (ht1632_data_port, ht1632_data_pin);
	// pulse wr
	clear_pin(ht1632_wr_port, ht1632_wr_pin);
	set_pin  (ht1632_wr_port, ht1632_wr_pin);

	// send 1
	set_pin  (ht1632_data_port, ht1632_data_pin);
	// pulse wr
	clear_pin(ht1632_wr_port, ht1632_wr_pin);
	set_pin  (ht1632_wr_port, ht1632_wr_pin);

	// send mem address of zero
	clear_pin(ht1632_data_port, ht1632_data_pin);
	
	// write mem addr, bits 6 -> 0
	for(count = 0 ; count < 7 ; count++) {
			
		// pulse wr
		clear_pin(ht1632_wr_port, ht1632_wr_pin);
		set_pin  (ht1632_wr_port, ht1632_wr_pin);
	}

	buffer_loc = 0;
	for (count = 0; count < 48; count++) {
		data = draw_buffer0[count];
		for (bit_loc = 0; bit_loc < 8; bit_loc++) {
			if (data.0) {
				set_pin(ht1632_data_port, ht1632_data_pin);
			} else {
				clear_pin(ht1632_data_port, ht1632_data_pin);
			}
			data = data >> 1;	
			clear_pin(ht1632_wr_port, ht1632_wr_pin);
			set_pin  (ht1632_wr_port, ht1632_wr_pin);
		}	
	}	
	
	// reset CS

	set_pin(ht1632_cs1_port, ht1632_cs1_pin);
}	
#endif

uns8 drv_get_pixel(uns8 x, uns8 y) {
	//ht1632_get_pixel(x, y);
}

void drv_clear_screen() {
	//ht1632_clear_screen();
}

void drv_setup_io() {
	ht1632_setup_io();
}

void drv_init() {
	// 2416 board is configured as 16 COMMONs
	ht1632_init(HT1632_CMD_PMOS_16_COMMON);
}	

#endif
