/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/
#include "ea_dsp7s04.h"
#include "pic_serial.h"

uns8 display[4];
uns8 dots;

void ea_dsp7s04_setup_io(void) {

	cat4016_setup_io();
	
}

void ea_dsp7s04_init(void) {
	
	display[0] = 0;
	display[1] = 0;
	display[2] = 0;
	display[3] = 0;
	dots = 0;
	
}	

uns8 ea_dsp7s04_translate(char in) {

uns8 res;
	
	switch (in) {
		//                       R  
		//                       TT
		case '$' : res = 0b00000010; break;
		case '0' : res = 0b11111100; break;
		case '1' : res = 0b01100000; break;
		case '2' : res = 0b11011010; break;
		case '3' : res = 0b11110010; break;
		case '4' : res = 0b01100110; break;
		case '5' : res = 0b10110110; break;
		case '6' : res = 0b10111110; break;
		case '7' : res = 0b11100000; break;
		case '8' : res = 0b11111110; break;
		case '9' : res = 0b11110110; break;
		case 'A' : res = 0b11101110; break;
		case 'b' : res = 0b00111110; break;
		case 'c' : res = 0b00011010; break;
		case 'C' : res = 0b10011100; break;
		case 'd' : res = 0b01111010; break;
		case 'E' : res = 0b10011110; break;
		case 'F' : res = 0b10001110; break;
		case 'G' : res = 0b10111100; break;
		case 'h' : res = 0b00101110; break;		
		case 'H' : res = 0b01101110; break;
		case 'I' : res = 0b01100000; break;
		case 'J' : res = 0b01110000; break;
		case 'L' : res = 0b00011100; break;
		case 'n' : res = 0b00101010; break;
		case 'N' : res = 0b11101100; break;
		case 'o' : res = 0b00111010; break;
		case 'O' : res = 0b11111100; break;
		case 'P' : res = 0b11001110; break;
		case 'r' : res = 0b00001010; break;
		case 'S' : res = 0b10110110; break;
		case 't' : res = 0b00011110; break;
		case 'u' : res = 0b00111000; break;
		case 'U' : res = 0b01111100; break;
		default  : res = 0b00000000; break;
	}	

	return res;
		
}	

void ea_dsp7s04_update_display(void) {

	cat4016_write_data(display[3] | (dots >> 3) , display[2] | ((dots >> 2) & 0x01));
	cat4016_write_data(display[1] | ((dots >> 1) & 0x01), display[0] | (dots & 0x01));
	cat4016_latch_data();
}


void ea_dsp7s04_set_display(char s, uns8 position) {
	
	if (position < 4) {
		display[position] = ea_dsp7s04_translate(s);
	}

}	

void ea_dsp7s04_set_dot(uns8 position) {
	
	dots = dots | (1 << position);
}	

void ea_dsp7s04_clear_dot(uns8 position) {
	
	dots = dots & ~(1 << position);
}	


void ea_dsp7s04_print_str(char *str) {

uns8 count;

	for( count = 0; count < 4; count++ ) {
		display[count] = ea_dsp7s04_translate(str[count]);
	}
	ea_dsp7s04_update_display();

}
	