/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file ea_ldp6416.h
    \brief Support routines for LDP-6416 LED display panel
	
*/

#ifndef __ea_ldp6416_h
#define __ea_ldp6416_h

#include "pic_utils.h"

#include "draw\draw.h"

#include "config.h"

#if ea_ldp6416_displays == 1
	// x_refresh 16 = 16*8=128 bits (64 pixels) 
	#define x_refresh 16
	#define y_refresh 16
#endif	

#if ea_ldp6416_displays == 2
	#if ea_ldp6416_display_orientation == HORIZONTAL
		#define y_refresh 16
		#define x_refresh 32
	#else
		// VERTICAL
		#define y_refresh 16
		#define x_refresh 16
	#endif
#endif

#if ea_ldp6416_displays == 3
	#if ea_ldp6416_display_orientation == HORIZONTAL
		#define y_refresh 16
		#define x_refresh 48
	#else
		// VERTICAL
		#define y_refresh 16
		#define x_refresh 16
	#endif
#endif

#if ea_ldp6416_displays == 4
	#if ea_ldp6416_display_orientation == HORIZONTAL
		#define y_refresh 16
		#define x_refresh 64
	#else
		// VERTICAL
		#define y_refresh 16
		#define x_refresh 16
	#endif
#endif


		
	

/* Include this in your config.h

// - - - - - - - - - - - - - - - - - - - - 
// LDP-6416 defines
// - - - - - - - - - - - - - - - - - - - - 


#define ea_ldp6416_en_port PORTA
#define ea_ldp6416_en_pin  0

#define ea_ldp6416_r1_port PORTA
#define ea_ldp6416_r1_pin  5

#define ea_ldp6416_g1_port PORTA
#define ea_ldp6416_g1_pin  5

#define ea_ldp6416_a_port PORTE
#define ea_ldp6416_a_pin  1

#define ea_ldp6416_b_port PORTE
#define ea_ldp6416_b_pin  1

#define ea_ldp6416_c_port PORTE
#define ea_ldp6416_c_pin  1

#define ea_ldp6416_d_port PORTE
#define ea_ldp6416_d_pin  1

#define ea_ldp6416_s_port PORTE
#define ea_ldp6416_s_pin  1

#define ea_ldp6416_l_port PORTE
#define ea_ldp6416_l_pin  1

// Starting at 0, number of iterations allowed for brightness
// Note that at '3', 0-3 gives 4 levels of brightness but 
// requires interrupts 4 times as often
// Set to 0 for no brightness control

#define ea_ldp6416_max_brightness 3



*/


#define ea_ldp6416_setup() ea_ldp6416_setup_io()
void ea_ldp6416_setup_io();
void ea_ldp6416_init();

// - - - - - - - - - - - - - - - - - - - - 
// Checks defines have been made

#ifndef ea_ldp6416_en_port
	#error "You haven't defined ea_ldp6416_en_port in your config.h!"
#endif
#ifndef ea_ldp6416_en_pin
	#error "You haven't defined ea_ldp6416_en_pin in your config.h!"
#endif
#ifndef ea_ldp6416_r1_port
	#error "You haven't defined ea_ldp6416_r1_port in your config.h!"
#endif
#ifndef ea_ldp6416_r1_pin
	#error "You haven't defined ea_ldp6416_r1_pin in your config.h!"
#endif
#ifndef ea_ldp6416_g1_port
	#error "You haven't defined ea_ldp6416_g1_port in your config.h!"
#endif
#ifndef ea_ldp6416_g1_pin
	#error "You haven't defined ea_ldp6416_g1_pin in your config.h!"
#endif
#ifndef ea_ldp6416_a_port
	#error "You haven't defined ea_ldp6416_a_port in your config.h!"
#endif
#ifndef ea_ldp6416_a_pin
	#error "You haven't defined ea_ldp6416_a_pin in your config.h!"
#endif
#ifndef ea_ldp6416_b_port
	#error "You haven't defined ea_ldp6416_b_port in your config.h!"
#endif
#ifndef ea_ldp6416_b_pin
	#error "You haven't defined ea_ldp6416_b_pin in your config.h!"
#endif
#ifndef ea_ldp6416_c_port
	#error "You haven't defined ea_ldp6416_c_port in your config.h!"
#endif
#ifndef ea_ldp6416_c_pin
	#error "You haven't defined ea_ldp6416_c_pin in your config.h!"
#endif
#ifndef ea_ldp6416_d_port
	#error "You haven't defined ea_ldp6416_d_port in your config.h!"
#endif
#ifndef ea_ldp6416_d_pin
	#error "You haven't defined ea_ldp6416_d_pin in your config.h!"
#endif
#ifndef ea_ldp6416_s_port
	#error "You haven't defined ea_ldp6416_s_port in your config.h!"
#endif
#ifndef ea_ldp6416_s_pin
	#error "You haven't defined ea_ldp6416_s_pin in your config.h!"
#endif
#ifndef ea_ldp6416_l_port
	#error "You haven't defined ea_ldp6416_l_port in your config.h!"
#endif
#ifndef ea_ldp6416_l_pin
	#error "You haven't defined ea_ldp6416_l_pin in your config.h!"
#endif


#endif