/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/** \file i2c_hw.h
	\brief Outputs i2c interfaces (clock+data)
	
	
	Covers standard i2c-like interfaces (clock + data) and
	Sure Electronics displays which are a little different
*/	

#ifndef __i2c_hw_h
#define __i2c_hw_h

#include "pic_utils.h"
#include "config.h"

// Include the following in your config.h:
/*
// - - - - - - - - - - - 
// i2c defines
// - - - - - - - - - - -
// This needs to be the same as the hardware

#define i2c_clk_port PORTB
#define i2c_data_port PORTB
#define i2c_clk_pin  6
#define i2c_data_pin  4

// - - - - - - - - - - -
*/



/**
	\brief Send a byte of data using software i2c
	
	Sends a byte of data MSB first, data only changes
	on clock low
*/
void i2c_write(uns8 data);

/**
	\brief Send a byte of data using software i2c
	
	Sends a byte of data LSB first, data only changes
	on clock low
*/
void i2c_write_lsb(uns8 data);

/**
	\brief Setup ports and pins for i2c output
	
	Setup ports and pins for i2c output
*/
void i2c_setup_io();


/** 
	\brief i2c test routine
	
*/
void i2c_pulse_0();
/** 
	\brief i2c test routine
	
*/
void i2c_pulse_1();


#ifndef i2c_clk_port
	#error "i2c_clk_port not defined in config.h"
#endif	
#ifndef i2c_data_port
	#error "i2c_data_port not defined in config.h"
#endif	
#ifndef i2c_clk_pin
	#error "i2c_clk_pin not defined in config.h"
#endif	
#ifndef i2c_data_pin
	#error "i2c_data_pin not defined in config.h"
#endif	



#endif