/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file its_mode2.h
    \brief ITS Mesh networking mode 2
	
*/

//!!! look at the adding in routes bit...

#ifndef its_mode2_h
#define its_mode2_h

#include "pic_utils.h"

/* Put this in your config.h

// - - - - - - - - - - - - - - - - - - - - 
// ITS mode2 defines
// - - - - - - - - - - - - - - - - - - - - 


#define ITS_MODE2_ROUTER
// or
//#define ITS_MODE2_DEVICE

#define ITS2_SEEN_LIST_SIZE 10
#define ITS2_TX_QUEUE_SIZE 5
#define ITS2_MAX_HOP_COUNT 5
#define ITS2_RESEND_TICK_DELAY 5000
#define ITS2_SEND_MAX_TRIES 2

*/

// positions in packet
#define POS_KEY1		0x00
#define POS_KEY2		0x01
#define POS_LENGTH_HEADER	0x02
#define POS_PKT_TYPE	0x03
#define POS_SEQUENCE	0x04
#define POS_NETWORK_L	0x05
#define POS_NETWORK_H	0x06
#define POS_SOURCE_L	0x07
#define POS_SOURCE_H	0x08
#define POS_DEST_L		0x09
#define POS_DEST_H		0x0a
#define POS_MAX_HOP_COUNT	0x0b
#define	POS_NUM_ROUTES	0x0c
#define POS_HOP_COUNT	0x0d
#define POS_ROUTE_START	0x0e

// Queue status of an individual packet
#define QS_WAITING_ON_LOCAL_ADDR	0x00
#define QS_WAITING_ON_NETWORK_ADDR	0x01
#define QS_READY_TO_SEND			0x02
#define QS_WAITING_ON_ACK			0x03
#define QS_ACK_RECEIVED				0x04
#define QS_SENT						0x05
#define QS_ROUTING_FAILED			0x06

// Remote indicator of address
#define REMOTE_DEVICE 0xffff

#define ITS2_TX_STATUS_SUCCESS	0x00
#define ITS2_TX_STATUS_NO_ROUTE		0x01
#define ITS2_TX_STATUS_NO_ACK	0x02
#define ITS2_TX_STATUS_REMOTE_NO_ACK	0x03
#define ITS2_TX_STATUS_NEXT_HOP_UNKNOWN	0x04

// Result of trying to route item
#define ITS2_UPDATE_ROUTE_SUCCESS	0x00
#define ITS2_UPDATE_ROUTE_FAIL		0x01

// Result of adding an item to the queue
#define ITS2_STATUS_QUEUED 			0x00
#define ITS2_STATUS_TX_QUEUE_FULL	0x01
#define ITS2_NO_AVAILABLE_SLOTS		0xff

// Settings of queue item flag
#define ITS2_FLAG_ACK 	0x01
#define ITS2_FLAG_NO_ACK 0x02
#define ITS2_FLAG_DELETED 0xff

// Debug settings for testing 
extern bit debug_module;

/** ITS2 Packet definition*/
typedef struct _its2_packet { 
	 uns8 packet_type;	/** Type of packet (see its_common.h) */
	 uns8 sequence;		/** Sequence of packet (incremented for each packet)*/
	 uns16 its_network_id;	/** Network ID of packet */
	 uns16 its_source_id;	/** ITS device ID of sender */
	 uns16 its_dest_id;		/** ITS device ID of destination */
	 uns8 max_hop_count;	/** Maximum number of permitted routes */
	 uns8 num_routes;		/** Number of routes (hops) specified for this packet to take */
	 uns8 hop_count;		/** Number of routes (hops) already made by this packet */
	 uns16 routers[ITS2_MAX_HOP_COUNT];
} its2_packet;

typedef struct _queued_item {
	its2_packet	packet;
	uns8		*data;
	uns8		data_length;
	uns16		dest_its_device_id;
	uns8		dest_device_handle;
    uns16     	tick_sent;
    uns8		sent_count;
    uns8		status;
    uns8		flag; // 0 don't resend / 1 resend / 0xff deleted
} queued_item;

typedef struct _seen_packet {
	uns8	sequence;
	uns16	its_source_id;
} seen_packet;


enum _its2_result { RESULT_SUCCESSFUL, RESULT_FAILED, ITEM_QUEUED, QUEUE_FULL,
	                ROUTING_TOO_MANY_HOPS, NEXT_HOP_NOT_LOCAL };
typedef enum _its2_result its2_result;

enum _its2_state {
	STATE_STARTUP, 
	STATE_RUNNING,
	STATE_SEARCHING,
	STATE_ASSOCIATED,
	STATE_UNASSOCIATED,
};

typedef enum _its2_state its2_state;

extern its2_state state;

/* routines for both router and device */

void its2_setup_io();
its2_result its2_find_coordinator();


/* router routines */

uns8 its2_router_init(uns16 my_device_id, uns16 network_id);
void its2_transmit_status_callback(its2_packet *pkt, uns8 status);
void its2_router_receive_callback(uns16 device_id, uns8 *data, uns8 data_length);
its2_result its2_router_queue_packet(uns16 device_id, uns8 packet_type, uns8 *data, uns8 data_length, uns8 ack);

void its2_router_process();



void its2_process_tx_queue();
void its2_request_local_addr(uns16 device_id);
void its2_respond_local_addr(uns16 device_id);
void its2_request_net_addr(uns16 device_id);
void its2_respond_net_addr(uns16 device_id);
its2_result its2_rebroadcast_net_discover_req(its2_packet *pkt);
its2_result its2_forward_routed_packet(its2_packet *pkt, uns8 *data, uns8 data_length);
void its2_print_queue();
void its2_delete_item_from_queue(queued_item *item);
void its2_print_packet(its2_packet *pkt);

/* internal */
its2_result its2_router_handle_association(uns16 pan_id, uns16 its_device_id);
void its2_transmit(queued_item *item);

/* Device specific routines */

its2_result its2_device_init(uns16 my_device_id, uns16 network_id);
void its2_device_receive_callback(uns8 *data, uns8 data_length);
its2_result its2_device_transmit(uns16 device_id, uns8 *data, uns8 data_length);
void its2_device_process();

#endif