/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "ms5540.h"
#include "pic_serial.h"
#include "debug.h"

// 2 us to give us 500khz
#define MS5540_DELAY_AMOUNT 2

int16 c1, c2, c3, c4, c5, c6;

void ms5540_setup_io(void) {
	
	clear_pin(ms5540_sclk_port, ms5540_sclk_pin);
	make_output(ms5540_sclk_port, ms5540_sclk_pin);
	make_output(ms5540_din_port, ms5540_din_pin);
	make_input(ms5540_dout_port, ms5540_dout_pin);
}

void ms5540_pulse_sclk() {

	set_pin(ms5540_sclk_port, ms5540_sclk_pin);
    delay_us(MS5540_DELAY_AMOUNT);
    clear_pin(ms5540_sclk_port, ms5540_sclk_pin);
    delay_us(MS5540_DELAY_AMOUNT);

}

void ms5540_send_start() {
	
	set_pin(ms5540_din_port, ms5540_din_pin);
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
}

void ms5540_send_stop() {
	
	clear_pin(ms5540_din_port, ms5540_din_pin);
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
}
	

void ms5540_reset() {

uns16 reset_word = 0b1010101010101010;
uns8 count;

    for( count = 1 ; count < 17 ; count++ )	// 1- 17 to get correct bit pattern
    {
        change_pin(ms5540_din_port, ms5540_din_pin, count.0);
		ms5540_pulse_sclk();
    }
	// five more pulses
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
	

}

 

uns16 ms5540_get_config(uns8 config_word) {

uns8 request;
uns8 count;
uns16 result;

	switch (config_word) {
		case 1:
			request = 0b01010100;
			break;
		case 2:
			request = 0b01011000;
			break;
		case 3:
			request = 0b01100100;
			break;
		case 4:
			request = 0b01101000;
			break;
	}
	ms5540_send_start();

    for( count = 0 ; count < 6 ; count++ )	// 1- 17 to get correct bit pattern
    {
        change_pin(ms5540_din_port, ms5540_din_pin, request.7);
		
		request = request << 1;
		
		ms5540_pulse_sclk();
    }
		
	ms5540_send_stop();
	
	// naked clock pulse
    ms5540_pulse_sclk();

	// okay now clock the word out
	for(count = 0 ; count < 16 ; count++)
    {
        ms5540_pulse_sclk();

        result = result << 1;
        result.0 = test_pin(ms5540_dout_port, ms5540_dout_pin);
	}

	// naked clock pulse
	ms5540_pulse_sclk();
	
	return result;

}   

uns16 ms5540_get_raw_pressure() {

uns8 setup_word = 0b1010;
uns8 count;
uns16 result;

	ms5540_send_start();
	// 1
	set_pin(ms5540_din_port, ms5540_din_pin);
	ms5540_pulse_sclk();
	// 0
	clear_pin(ms5540_din_port, ms5540_din_pin);
	ms5540_pulse_sclk();
	// 1
	set_pin(ms5540_din_port, ms5540_din_pin);
	ms5540_pulse_sclk();
	// 0
	clear_pin(ms5540_din_port, ms5540_din_pin);
	ms5540_pulse_sclk();
	
	ms5540_send_stop();
	
	// two more pulses
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
	
	// now wait for conversion to finish (din goes low)
	
	while (test_pin(ms5540_dout_port, ms5540_dout_pin) != 0) {
	}
	
	// okay now clock the word out
	for(count = 0 ; count < 16 ; count++)
    {
        ms5540_pulse_sclk();

        result = result << 1;
        result.0 = test_pin(ms5540_dout_port, ms5540_dout_pin);
	}

	// naked pulse
	ms5540_pulse_sclk();
	return result;
	
}
	
uns16 ms5540_get_raw_temp() {

uns8 setup_word = 0b1010;
uns8 count;
uns16 result;


	ms5540_send_start();
	// 1
	set_pin(ms5540_din_port, ms5540_din_pin);
	ms5540_pulse_sclk();
	// 0
	clear_pin(ms5540_din_port, ms5540_din_pin);
	ms5540_pulse_sclk();
	// 0
	ms5540_pulse_sclk();
	// 1
	set_pin(ms5540_din_port, ms5540_din_pin);
	ms5540_pulse_sclk();
	
	ms5540_send_stop();
	
	// two more pulses
	ms5540_pulse_sclk();
	ms5540_pulse_sclk();
	
	// now wait for conversion to finish (din goes low)
	
	while (test_pin(ms5540_dout_port, ms5540_dout_pin) != 0) {
	}
	
	// okay now clock the word out
	for(count = 0 ; count < 16 ; count++)
    {
        ms5540_pulse_sclk();

        result = result << 1;
        result.0 = test_pin(ms5540_dout_port, ms5540_dout_pin);
	}

	// naked pulse
	ms5540_pulse_sclk();
	
	return result;
	
}

void ms5540_init() {

uns16 word1, word2, word3, word4, x, y;
/*c1=22801
c2=2256
c3=716
c4=692
c5=872
c6=26	

*/
	// reset
	ms5540_reset();

	// get config
	word1 = ms5540_get_config(1);
	serial_print_int(word1);
	serial_print_spc();
	word2 = ms5540_get_config(2);
	serial_print_int(word2);
	serial_print_spc();
	word3 = ms5540_get_config(3);
	serial_print_int(word3);
	serial_print_spc();
	word4 = ms5540_get_config(4);
	serial_print_int(word4);
	serial_print_nl();

	// parse config words
	
	c1 = (word1 >> 1) & 0x7fff;
	
	/*c5 = word2 >> 6;
	if (test_bit(word1, 0)) {
		set_bit(c5, 10);
	}
	*/
	x = (word1 << 10) & 0x0400;
	y = (word2 >> 6)  & 0x03ff;
	c5 = x | y;
	
	c6 = word2 & 0b0000000000111111; // grab last 6 bits of word2 as c6
	
	c4 = (word3 >> 6) & 0x03ff;
	
	x = (word3 << 6) &  0x0fc0;
	y = (word4 & 0b0000000000111111);
	c2 = x | y;
	
	c3 = (word4 >> 6) & 0x03ff;
	
	serial_print_str("c1=");
	serial_print_int(c1);
	serial_print_nl();
	serial_print_str("c2=");
	serial_print_int(c2);
	serial_print_nl();
	serial_print_str("c3=");
	serial_print_int(c3);
	serial_print_nl();
	serial_print_str("c4=");
	serial_print_int(c4);
	serial_print_nl();
	serial_print_str("c5=");
	serial_print_int(c5);
	serial_print_nl();
	serial_print_str("c6=");
	serial_print_int(c6);
	serial_print_nl();

	
}
	
	
	
void ms5540_calc_temp_and_pressure() {
	
int16 d1, d2, ut1, off;
int16 temp, p, dt;
int32 sens, x;

//	c1 = 23470;
//	c2 = 1324;
//	c3 = 737;
//	c4 = 393;
//	c5 = 628;
//	c6 = 25;

	
	d1 = ms5540_get_raw_pressure();
//	d1 = 16460;
	
	serial_print_str("get pr d1: ");
	serial_print_int(d1);
	serial_print_nl();

	d2 = ms5540_get_raw_temp();
//	d2 = 27856;
	serial_print_str("get temp: ");
	serial_print_int(d2);
	serial_print_nl();
	
	ut1 = 8*c5+20224;
	
	serial_print_str("ut1= ");
	serial_print_int(ut1);
	serial_print_nl();
	
	dt = d2 - ut1;
	
	serial_print_str("dt= ");
	serial_print_int(dt);
	serial_print_nl();
	
	temp = 200 + (int32)dt * ((int32)c6 + 50) / 1024;
	
	serial_print_str("temp= ");
	serial_print_int(temp);
	serial_print_nl();
	
	off = c2 * 4 + (((int32)c4-512)*dt) / 4096;

	serial_print_str("off= ");
	serial_print_int(off);
	serial_print_nl();
	 
	sens = c1 + ((int32)c3*dt)/1024 + 24576;

	serial_print_str("sens= ");
	serial_print_int(sens);
	serial_print_nl();

	x = ((int32)sens * ((int32)d1-7168))/16384 - (int32)off;
	
	serial_print_str("x= ");
	serial_print_int(x);
	serial_print_nl();
	
	p = x * 10/32 + 250*10;
	
	serial_print_str("p= ");
	serial_print_int(p);
	serial_print_nl();
}	

	
