/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "ms5511.h"
#include "pic_serial.h"
#include "debug.h"
#include "i2c.h"


#define MS5511_RESET		0x1e
#define MS5511_PROM_READ	0xa0

int16 c1, c2, c3, c4, c5, c6;

void ms5511_setup_io(void) {
	
	i2c_setup_io();
	
}

	

void ms5511_reset() {

	i2c_start();
	i2c_send_byte(MS5511_ADDR << 1);
	i2c_send_byte(MS5511_RESET);
	i2c_stop();

}

 

uns16 ms5511_get_config(uns8 config_word) {

return i2c_read_eeprom_16bit(MS5511_ADDR << 1, MS5511_PROM_READ + (config_word << 1));

uns8 r1, r2;

	i2c_start();
	i2c_send_byte(MS5511_ADDR << 1);
	i2c_send_byte(MS5511_PROM_READ + (config_word << 1));
	i2c_stop();
	
	i2c_start();
	i2c_send_byte((MS5511_ADDR << 1) + 1);	 // read
	
	r1 = i2c_receive_byte();
	serial_print_str("r1=");
	serial_print_int_hex(r1);
	serial_print_nl();
	
	r2 = i2c_receive_byte();
	serial_print_str("r2=");
	serial_print_int_hex(r2);
	serial_print_nl();
	
	return (r1 << 8) + r2;
	

}   

uns32 ms5511_get_raw_pressure() {

uns32 result;

	return result;
	
}
	
uns32 ms5511_get_raw_temp() {

}

void ms5511_init() {
uns16 c0, c7;
	// reset
	ms5511_reset();

	// get config
	c0 = ms5511_get_config(0);
	
	serial_print_str("C1\n");
	c1 = ms5511_get_config(1);
	serial_print_str("C2\n");
	c2 = ms5511_get_config(2);
	serial_print_str("C3\n");
	c3 = ms5511_get_config(3);
	serial_print_str("C4\n");
	c4 = ms5511_get_config(4);
	serial_print_str("C5\n");
	c5 = ms5511_get_config(5);
	serial_print_str("C6	\n");
	c6 = ms5511_get_config(6);
	c7 = ms5511_get_config(7);

	serial_print_str("c0=");
	serial_print_int(c0);
	serial_print_nl();
	serial_print_str("c1=");
	serial_print_int(c1);
	serial_print_nl();
	serial_print_str("c2=");
	serial_print_int(c2);
	serial_print_nl();
	serial_print_str("c3=");
	serial_print_int(c3);
	serial_print_nl();
	serial_print_str("c4=");
	serial_print_int(c4);
	serial_print_nl();
	serial_print_str("c5=");
	serial_print_int(c5);
	serial_print_nl();
	serial_print_str("c6=");
	serial_print_int(c6);
	serial_print_nl();
	serial_print_str("c7=");
	serial_print_int(c7);
	serial_print_nl();

	
}
	
	
	
void ms5511_calc_temp_and_pressure() {
}	

uns8 ms5511_calc_crc(uns8 n_prom[])
{
	int cnt; // simple counter
	uns16 n_rem; // crc reminder
	uns16 crc_read; // original value of the crc
	uns8 n_bit;
	
	n_rem = 0x00;
	crc_read = n_prom[7]; //save read CRC
	n_prom[7] = (0xFF00 & (n_prom[7])); //CRC byte is replaced by 0
	
	for (cnt = 0; cnt < 16; cnt++) { // operation is performed on bytes
	
		 // choose LSB or MSB
		if (cnt%2==1) n_rem ^= (unsigned short) ((n_prom[cnt>>1]) & 0x00FF);
		else n_rem ^= (unsigned short) (n_prom[cnt>>1]>>8);
		
		for (n_bit = 8; n_bit > 0; n_bit--) {
		
			if (n_rem & (0x8000)) {
				n_rem = (n_rem << 1) ^ 0x3000;
			} else {
				n_rem = (n_rem << 1);
			}
		}
	}
	n_rem = (0x000F & (n_rem >> 12)); // // final 4-bit reminder is CRC code
	n_prom[7] = crc_read; // restore the crc_read to its original place

	return (n_rem ^ 0x00);

}
	
