/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "config.h"
#include "platform_leds.h"
#include "platform.h"
#include "pic_tick.h"

#ifndef PLATFORM_LEDS_FLASH_TICKS
	#define PLATFORM_LEDS_FLASH_TICKS 250
#endif	

#ifdef led1_port
	uns16 led1_start_tick;
	bit led1_on = 0;
	bit led1_flashing = 0;
#endif

#ifdef led2_port
	uns16 led2_start_tick;
	bit led2_on = 0;
	bit led2_flashing = 0;
#endif

#ifdef led3_port
	uns16 led3_start_tick;
	bit led3_on = 0;
	bit led3_flashing = 0;
#endif

void platform_leds_setup_io() {			

	#ifdef led1_port
		clear_pin(led1_port, led1_pin);
		make_output(led1_port, led1_pin);
	#endif
	#ifdef led2_port
		clear_pin(led2_port, led2_pin);
		make_output(led2_port, led2_pin);
	#endif
	#ifdef led3_port
		clear_pin(led3_port, led3_pin);
		make_output(led3_port, led3_pin);
	#endif
}		

void platform_leds_on(uns8 led) {
		switch (led) {
		case 1:
			platform_led1_on();
			break;
		#ifdef led2_port
			case 2:
				platform_led2_on();
				break;
		#endif
		#ifdef led3_port		
			case 3:
				platform_led3_on();
				break;
		#endif
	}
}

void platform_leds_off(uns8 led) {
		switch (led) {
		case 1:
			platform_led1_off();
			break;
		#ifdef led2_port
			case 2:
				platform_led2_off();
				break;
		#endif
		#ifdef led3_port		
			case 3:
				platform_led3_off();
				break;
		#endif
	}
}

void platform_leds_flash(uns8 led) {
	
	switch (led) {
		case 1:
			platform_led1_on();
			led1_start_tick = tick_get_count();
			led1_on = 1;
			break;
		#ifdef led2_port
			case 2:
				platform_led2_on();
				led2_start_tick = tick_get_count();
				led2_on = 1;
				break;
		#endif
		#ifdef led3_port		
			case 3:
				platform_led3_on();
				led3_start_tick = tick_get_count();
				led3_on = 1;
				break;
		#endif
	}
}

void platform_leds_flashing(uns8 led, uns8 enable) {
	switch (led) {
		case 1:
			led1_flashing = enable.0;
			break;
		#ifdef led2_port
			case 2:
				led2_flashing = enable.0;
				break;
		#endif
		#ifdef led3_port		
			case 3:
				led3_flashing = enable.0;
				break;
		#endif
	}
}	

void platform_leds_process() {

uns16 current_tick;

	current_tick = tick_get_count();
	
	#ifdef led1_port
	if (led1_on || led1_flashing) {
		if (tick_calc_diff(led1_start_tick, current_tick) > PLATFORM_LEDS_FLASH_TICKS) {
			if (led1_on) {
				led1_on = 0;
				platform_led1_off();
			} else {
				led1_on = 1;
				platform_led1_on();
			}	
			if (led1_flashing) {
				led1_start_tick = tick_get_count();	
			}	
		}
	}
	#endif
	#ifdef led2_port
	if (led2_on || led2_flashing) {
		if (tick_calc_diff(led2_start_tick, current_tick) > PLATFORM_LEDS_FLASH_TICKS) {
			if (led2_on) {
				led2_on = 0;
				platform_led2_off();
			} else {
				led2_on = 1;
				platform_led2_on();
			}	
			if (led2_flashing) {
				led2_start_tick = tick_get_count();	
			}	
		}
	}
	#endif
	#ifdef led3_port
	if (led3_on || led1_flashing) {
		if (tick_calc_diff(led3_start_tick, current_tick) > PLATFORM_LEDS_FLASH_TICKS) {
			if (led3_on) {
				led3_on = 0;
				platform_led3_off();
			} else {
				led3_on = 1;
				platform_led3_on();
			}	
			if (led3_flashing) {
				led3_start_tick = tick_get_count();	
			}	
		}
	}
	#endif
}
