/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "soundout.h"
#include "pic_serial.h"


void soundout_setup_io() {
	
	make_output(soundout_clk_port, soundout_clk_pin);
	set_pin(soundout_clk_port, soundout_clk_pin);	// idle high
	
	make_output(soundout_data_port, soundout_data_pin);
	// don't care about data pin state
	#ifdef soundout_reset_port
		make_output(soundout_reset_port, soundout_reset_pin);
		set_pin(soundout_reset_port, soundout_reset_pin);
	#endif
	#ifdef soundout_busy_port
		make_input(soundout_busy_port, soundout_busy_pin);
	#endif
}	

void soundout_reset() {
	#ifdef soundout_reset_port 
		clear_pin(soundout_reset_port, soundout_reset_pin);
		delay_ms(10);	// minimum 5ms
		set_pin(soundout_reset_port, soundout_reset_pin);
	#endif
}		
		
void soundout_standby() {
	#ifdef soundout_reset_port 
		clear_pin(soundout_reset_port, soundout_reset_pin);
		// Need to wait for 1 second to go into standby
		// seems a shame to hang around here waiting though...
		// delay_s(1);
	#endif
}	
void soundout_wake() {
	#ifdef soundout_reset_port 
		set_pin(soundout_reset_port, soundout_reset_pin);
	#endif
}	

void soundout_send_data(uns16 data) {
	
	// Signal start
	clear_pin(soundout_clk_port, soundout_clk_pin);
	delay_ms(3);	// tSTART = 2ms
	
	for (uns8 count = 0; count < 16; count++) {
		if (data.15) {
			set_pin(soundout_data_port, soundout_data_pin);
		} else {
			clear_pin(soundout_data_port, soundout_data_pin);
		}
		delay_us(2); 	// tDS = 1us
		set_pin(soundout_clk_port, soundout_clk_pin);
		delay_us(220);	// tCH = 200us
		clear_pin(soundout_clk_port, soundout_clk_pin);
		delay_us(220); // tCL = 200us
		data = data << 1;
	}
	// Signal end
	set_pin(soundout_clk_port, soundout_clk_pin);
	delay_ms(3);	// tSTOP = 2ms
}

void soundout_set_file_id(uns16 file_id) {

	soundout_send_data(file_id);
}

void soundout_set_volume(uns8 level) {
	level = level & 0x07;
	soundout_send_data(soundout_VOLUME_CMD + level);
}
	
void soundout_play_pause() {
	soundout_send_data(soundout_PLAY_PAUSE_CMD);
}
	
void soundout_stop() {

	soundout_send_data(soundout_STOP_CMD);

}

uns8 soundout_is_busy() {
	#ifdef soundout_busy_port 
		return test_pin(soundout_busy_port, soundout_busy_pin);
	#else
		return 0;
	#endif	
}