/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file soundout.h
    \brief SoundOut MOD-1007 (Somo-14D) audio player interface

	Library for accessing the functionality of the WTV .ad4 audio player

Put the following into your config.h

// - - - - - - - - - - - - - - - - - - - - 
// SoundOut defines
// - - - - - - - - - - - - - - - - - - - - 

#define soundout_clk_port PORTA
#define soundout_clk_pin 1

#define soundout_data_port PORTA
#define soundout_data_pin 2

// don't define these if you don't want to use them

#define soundout_reset_port PORTA
#define soundout_reset_pin 3

#define soundout_busy_port PORTA
#define soundout_busy_pin 4

// - - - - - - - - - - - - - - - - - - - - 

*/

#ifndef __soundout_h
#define __soundout_h
#include "pic_utils.h"

#define soundout_PLAY_PAUSE_CMD 0xfffe
#define soundout_STOP_CMD		0xffff
#define soundout_VOLUME_CMD		0xfff0

void soundout_setup_io();
void soundout_reset();
void soundout_standby();
void soundout_wake();
void soundout_set_file_id(uns16 file_id);
void soundout_set_volume(uns8 level);
void soundout_play_pause();
void soundout_stop();
uns8 soundout_is_busy();

#ifndef soundout_clk_port
	#error "You haven't defined soundout_clk_port in your config.h!"
#endif

#ifndef soundout_data_port
	#error "You haven't defined soundout_data_port in your config.h!"
#endif

#ifndef soundout_clk_pin
	#error "You haven't defined soundout_clk_pin in your config.h!"
#endif

#ifndef soundout_data_port
	#error "You haven't defined soundout_clk_port in your config.h!"
#endif

#endif
