/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/** \file sure_7seg.h
 	\brief Routines to talk to Sure electronics seven segment displays


*/

#ifndef __sure_7seg_h
#define __sure_7seg_h

#include "spi.h"
#include "pic_utils.h"

/**
	\brief Setup ports and pins to communicate to Sure display
	
	Set up ports and pins as appropriate to communicate
	via SPI to Sure 7 segment displays
*/
void sure_7seg_setup();

/**
	\brief Display ASCII string to 7 segment displays
	
	Converts ASCII to the appropriate magic characters
	to display on a Sure 7 segment display. Only numbers
	0-9 and space are implented for now.
 
 To create your own characters, add together:
       0x40
     --------
     |      |
0x02 |      | 0x20
     | 0x01 |
     |------|
     |      |
0x04 |      | 0x10
     |      |
     --------
      0x08

*/
void sure_7seg_write_str(char *data);

#endif
