//-----------------------------------------------------
// Pic Pack library
// 
// DSP-0801 Firmware
//
//
// Ian Harris 2011
// www.embeddedadventures.com
//
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "ccld.h"

// System includes
#include "string.h"
#include "stdlib.h"
#include "config.h"
#include "pic_term.h"

#include "spi_hw.h"

#include "pic_tick.h"
#include "pic_timer.h"
#include "pic_timer1.h"
#include "ea_dsp0801.h"

uns8 display[16];
uns8 buffer[16];

uns8 dim = 0;

// do bit patterns to test display

uns8 self_test_mode = 0;


uns8 current_pair = 3;

void display_refresh() {

	current_pair++;

	if (current_pair == 4) {
		current_pair = 0;
	}	
	

	ccld_write_data_byte(display[current_pair*2+8]);
	ccld_write_data_byte(display[current_pair*2+9]);
	ccld_write_data_byte(display[current_pair*2]);
	ccld_write_data_byte(display[current_pair*2+1]);


	set_pin(dsp0801_d1_port, dsp0801_d1_pin);
	set_pin(dsp0801_d2_port, dsp0801_d2_pin);
	set_pin(dsp0801_d3_port, dsp0801_d3_pin);
	set_pin(dsp0801_d4_port, dsp0801_d4_pin);

	ccld_latch_data();
	
	switch (current_pair) {
		case 0: clear_pin(dsp0801_d1_port, dsp0801_d1_pin); break;
		case 1: clear_pin(dsp0801_d2_port, dsp0801_d2_pin); break;
		case 2: clear_pin(dsp0801_d3_port, dsp0801_d3_pin); break;
		case 3: clear_pin(dsp0801_d4_port, dsp0801_d4_pin); break;
	}

		
}


void reset_reason() {
	
	serial_print_str("Reset: ");
	if (!test_bit(rcon, RI)) {
		serial_print_str("Reset instruction ");
	}
	if (!test_bit(rcon, PD)) {
		serial_print_str("Sleep / powerdown ");
	}
	if (!test_bit(rcon, TO)) {
		serial_print_str("WDT timeout ");
	}
	if (!test_bit(rcon, BOR)) {
		serial_print_str("BOR ");
		set_bit(rcon, BOR);
	}
	if (!test_bit(rcon, POR)) {
		serial_print_str("POR ");
		set_bit(rcon, POR);
	}
	serial_print_nl();
}		


// Interrupt routine - - - - - - - - - -


uns8 got_spi = 0;
uns8 got_latch = 0;
uns8 buffer_end = 0;
uns8 next_byte;

void latch_buffer(void) {
uns8 count;
uns8 pos = buffer_end;

	for (count = 0; count < 16; count++) {

		display[count] = buffer[pos];
		if (pos == 15) {
			pos = 0;
		} else {
			pos++;
		}
		
	}
	//serial_putc('L');
		
}

void process_spi(void) {
uns8 new_spi, buffer_send;

		new_spi = sspbuf;
		sspbuf = next_byte;

		
		buffer[buffer_end] = new_spi;

		if (buffer_end == 15) {
			buffer_end = 0;
		} else {
			buffer_end++;
		}	
		buffer_send = buffer_end + 1;
		if (buffer_send == 16) {
			buffer_send = 0;
		}
		next_byte = buffer[buffer_send];
}

void lowi(void) {
	timer_handle_1_isr();

	serial_handle_tx_isr();
	serial_handle_rx_isr();

	timer_handle_0_isr();	

}

void interrupt() {

	while (test_bit(pir1, SSPIF)) {
		clear_bit(pir1, SSPIF);
		process_spi();

	}
	if (test_bit(intcon, INT0IF)) {
		clear_bit(intcon, INT0IF);
		latch_buffer();
	}	
}

void interrupt_low() {
		lowi();
}		

bit flip_bit = 0;

void timer_1_callback() {

	if (flip_bit) {
		display_refresh();
	} else if (dim) {
		set_pin(dsp0801_d1_port, dsp0801_d1_pin);
		set_pin(dsp0801_d2_port, dsp0801_d2_pin);
		set_pin(dsp0801_d3_port, dsp0801_d3_pin);
		set_pin(dsp0801_d4_port, dsp0801_d4_pin);
	}
		flip_bit = ~flip_bit;
}


// term_entry_callback will be called when 
// user pressed <enter> on serial "terminal".
// term_buffer contains the characters (if any)
// that the user has entered.

void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
uns8 count;	
uns16 digit;

	first_char = term_buffer[0];

	switch (first_char) {
		case 'p':
			count = 0;
			while ((term_buffer[count+1] != 0) && (count < 8)) {
				digit = ea_dsp0801_translate(term_buffer[count+1]);
				display[count*2] = digit >> 8;
				display[count*2+1] = digit & 0xff;
				count++;
			}
			break;

		case 'c':
			// print buffer (for debugging purposes!)
			serial_print_str("\nB:");
			for (count = 0; count < 16; count++) {
				serial_print_int_hex(buffer[count]);
				serial_print_spc();
			}
			serial_print_str(" BE:");
			serial_print_int(buffer_end);
			serial_print_nl();	
			serial_print_str("D:");
			for (count = 0; count < 16; count++) {
				serial_print_int_hex(display[count]);
				serial_print_spc();
			}

			serial_print_nl();	
			break;
		case 'd':
			dim = 1;
			break;
		case 'b':
			dim = 0;
			break;
		case '!':
			self_test_mode = !self_test_mode;
			break;
		default:
			serial_print_str("Unknown command.\n");
			break;
	}	// switch		

}	// term_entry_callback

void dsp0801_firmware_setup_io() {
	
	set_pin(dsp0801_d1_port, dsp0801_d1_pin);
	set_pin(dsp0801_d2_port, dsp0801_d2_pin);
	set_pin(dsp0801_d3_port, dsp0801_d3_pin);
	set_pin(dsp0801_d4_port, dsp0801_d4_pin);
	
	make_output(dsp0801_d1_port, dsp0801_d1_pin);
	make_output(dsp0801_d2_port, dsp0801_d2_pin);
	make_output(dsp0801_d3_port, dsp0801_d3_pin);
	make_output(dsp0801_d4_port, dsp0801_d4_pin);
	
}


// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {

uns8 count;
	
	// clear_bit(intcon2, RABPU);
	// set_bit(wpub, WPUB4);
	// set_bit(wpub, WPUB6);
	
	// turn off interrupts just in case
	kill_interrupts();
	
	// kill those pesky analogue inputs
	turn_analog_inputs_off();	
	
	// set serial to run at 115200bps 
	serial_setup(SPBRG_115200);
	
	// initialise termainl
	term_init();
	
	// Timer 0 is used for ticks - 1ms at 32MHz (8mips)
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 8000 -1); 

	// Timer 1 is used for led board refresh
	timer_setup_1(TIMER1_PRESCALER_OFF, 0xffff - 15000 -1);

	dsp0801_firmware_setup_io();
	
	ccld_setup_io();
	
	// first check if we are in self test
	
	// SIN is RB4
	make_input(PORTB, 4);
	// SOUT is RC7
	make_output(PORTC, 7);
	
	set_pin(PORTC, 7);
	delay_us(10);
	if (test_pin(PORTB, 4)) {
		clear_pin(PORTC, 7);
		delay_us(10);
		if (!test_pin(PORTB, 4)) {
			set_pin(PORTC, 7);
			delay_us(10);
			if (test_pin(PORTB,4)) {
				self_test_mode = 1;
			}
		}
	}
				
	
	for (count = 0; count < 16; count++) {
		buffer[count] = 0;
		display[count] = 0;
	}	

	if (!self_test_mode) {
	
		spi_hw_setup_io();
		spi_hw_init();

		clear_bit(pir1, SSPIF);	// clear interrupt
		set_bit(pie1, SSPIE);	// request interrupt
	
		sspbuf = 0;	// preload with 0
	
	}
	
	// make_input(PORTA, 0);	// due to hardware bug v5
	make_input(PORTC, 0);	// where Latch is connected to (INT0)
	
	// latch pin is connected to int0
	clear_bit(intcon, INT0IF);
	set_bit(intcon2, INTEDG0);
	set_bit(intcon, INT0IE);
	
	// set serial port, timers to low priority
	
	clear_bit(ipr1, TXIP);
	clear_bit(ipr1, RCIP);
	clear_bit(ipr1, TMR1IP);
	clear_bit(intcon2, TMR0IP);
	
	// enable low priority interrupts
	
	set_bit(rcon, IPEN);
	
	// turn peripheral interrupts on (ie, serial port)
	
	turn_peripheral_ints_on();
	
	// turn all interrupts on
	// turn_global_ints_on();
	set_bit(intcon, GIEH);
	set_bit(intcon, GIEL);
	
}



void main() {
	
	configure_system();
	clear_bit(rcon, SBOREN);
	
	delay_ms(100);	// mostly so screamer bootloader has time to switch windows

	timer_start_0();
	timer_start_1();

	serial_print_str("\n\nEmbedded Adventures DSP-0801 Firmware\n");
	serial_print_str( "\n<");
	
	// Print the time and date this file was compiled. Handy to know which
	// version you are running.
	
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");
	
	serial_print_str(FIRMWARE_VERSION);
	serial_print_nl();
	
	reset_reason();

	if (!self_test_mode) {
		serial_print_str("Commands:\n\npSTRING - print string\nd - dim\nb - bright\n\n");
	} else {
		serial_print_str("\n\n - - - - SELF TEST MODE - - - -\n\n");
	}	
	// Turn off the LED
	make_output(PORTC, 5);
	
	if (!self_test_mode) {
		clear_pin(PORTC, 5);
	} else {
		set_pin(PORTC, 5);
	}	
	
	
	uns16 tick = tick_get_count();
	uns16 current_tick;
	uns16 test_display_byte = 1;
	uns8 count;
	
	for(;;) {

		term_process();
		
		if (test_bit(sspcon1, SSPOV)) {
			serial_print_str("<OV>");
			clear_bit(sspcon1, SSPOV);
		}
		if (self_test_mode) {
			current_tick = tick_get_count();
			if (tick_calc_diff(tick, current_tick) > 100) {
				tick = current_tick;
				for (count = 0; count < 8; count++) {
					buffer[count*2] = test_display_byte >> 8;
					buffer[count*2+1] = test_display_byte & 0xff;
				}
				test_display_byte = test_display_byte << 1;
				if (test_display_byte == 0) {
					test_display_byte = 1;
				}
				latch_buffer();
			}	// time to change the display
		}	// self_test_mode				
	}	// never ending for loop
				
	
}	// main

