/*

ea_plt1003_usb_keyboard.c

Keyboard demo based on the EmbeddedAdventures PLT-1003 USB to Serial platform.

Since the PLT-1003 only has two buttons, we just make the keyboard pg up and pg dn.

*/


// System includes
#include "string.h"

// Config
#include "config.h"


// General PIC utilities
#include "pic_utils.h"

// Timer 0 functions
#include "pic_timer.h"

// Tick functions (builds on timer functions)
#include "pic_tick.h"

// Serial port communications
#include "pic_serial.h"

// Terminal functions (builds on serial port)
#include "pic_term.h"

// Platform LEDs
#include "platform_leds.h"

// USB general routines
#include "pic_usb.h"


uns16	tick_marker = 0;

uns8	up_count,
		down_count,
		right_count,
		left_count,
		center_count,
		check_count;

signed char buffer[8];

// Interrupt routine - - - - - - - - - -

void interrupt() {
	
		serial_handle_tx_isr();
		serial_handle_rx_isr();
		usb_handle_isr();	// USB priority isn't so important in this demo
		timer_handle_0_isr();
}

void usb_ep_data_out_callback(uns8 end_point, uns8 *buffer_location, uns16 byte_count) {
	serial_print_str("Got stuff! ");
	serial_print_int(byte_count);
	serial_print_str(" bytes\n");
}

void usb_ep_data_in_callback(uns8 end_point, uns16 byte_count) {
}

// configure_system

void configure_system() {
	
	// Turn off interupts just in case
	kill_interrupts();

	// Kill those pesky analogue inputs
	turn_analog_inputs_off();
	
	// Set serial ready to go at 115k	
	serial_setup(SPBRG_115200);

	// Use 0xffff - 4000 if you're using techtoys board at 16Mhz (1ms)
	// PLT-1003 runs at 48Mhz so we need bigger number
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 12000); 
	
	// Initialise the terminal library
	term_init();
	
	// Configure our pins for inputs
	#ifdef PGUP_PIN
		make_input(KB_PORT, PGUP_PIN); 
	#endif
	#ifdef PGDN_PIN
		make_input(KB_PORT, PGDN_PIN);
	#endif

	
	#ifdef NEED_WEAK_PULLUPS
		clear_bit(intcon2, RBPU); // turn on weak pull-ups
	#endif
	
	// Get USB subsystem ready
	usb_setup();

	// Turn on peripheral interrupts
	turn_peripheral_ints_on();
	
	// Turn on USB interrupts 
	turn_usb_ints_on();
	
	// Turn on all interrupts
	turn_global_ints_on();
}



void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
	
	first_char = term_buffer[0];

	switch (first_char) {
	}
}						


void keyboard_process() {

uns16 test_tick;

	test_tick = tick_get_count();	// find out what we're up to
	if (tick_calc_diff(tick_marker, test_tick) > 2) {	// every 3 ms
		tick_marker = test_tick;
		#ifdef UP_PIN
			if (!test_pin(KB_PORT, PGUP_PIN)) {
				up_count++;
			}
		#endif
		#ifdef DOWN_PIN		
			if (!test_pin(KB_PORT, PGDN_PIN)) {
				down_count++;
			}
		#endif
		check_count++;
		if (check_count == 5) {
			check_count = 0;
			if (usb_get_state() == st_CONFIGURED) {
				//Load the HID buffer
				buffer[0] = 0;
				buffer[1] = 0;
				if (up_count) {
					buffer[2] = 0x4b;
				} else {
					buffer[2] = 0x00;
				}	
				if (down_count) {
					buffer[3] = 0x4e;
				} else {
					buffer[3] = 0x00;
				}	
				buffer[4] = 0;
				buffer[5] = 0;
				buffer[6] = 0;
				buffer[7] = 0;
				
				usb_send_data(1, (uns8 *)&buffer, 8, /*first*/ 0); // ep 1

			}
			up_count = down_count = 0;
		}	
	}	
}

void main() {

		
	configure_system();
	
	delay_ms(100);
	serial_print_str("\n\nEA PLT-1003 PIC USB Keyboard demo\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	timer_start_0();	// kick that timer off...
	
	left_count = right_count = up_count = down_count = center_count = check_count = 0;
	serial_print_str("Attaching to USB...");
	
	usb_enable_module();
	
	while (usb_get_state() != st_CONFIGURED);
	
	serial_print_str("Done!\n");

	for(;;) {

		term_process();
		keyboard_process();

	}
	// main
}