//-----------------------------------------------------
// Pic Pack library
// 
// simon_lights.c
//
// Lights for the Simon game
//
// Ian Harris 2009
// imharris [at] gmail.com
//
// Released under the "do whatever you like with this
// but if it breaks, you get to keep both pieces.
//-----------------------------------------------------

#include "simon_lights.h"
#include "pic_serial.h"	

bit flashing  = 0;
uns8 flashing_cols = 0;
#define flash_red	0
#define flash_blue	1
#define flash_green 2
#define flash_yellow 3
#define flash_white 4
bit flashing_is_on = 0;
uns8 flashing_cycle = 0;
enum_lights_state lights_state;

void simon_lights_print() {
	serial_putc('<');
	if (test_output_pin(red_light_port, red_light_pin)) {
		serial_putc('R');
	} else {
		serial_putc(' ');
	}
	if (test_output_pin(green_light_port, green_light_pin)) {
		serial_putc('G');
	} else {
		serial_putc(' ');
	}
	if (test_output_pin(blue_light_port, blue_light_pin)) {
		serial_putc('B');
	} else {
		serial_putc(' ');
	}
	if (test_output_pin(yellow_light_port, yellow_light_pin)) {
		serial_putc('Y');
	} else {
		serial_putc(' ');
	}
	serial_putc('>');	
}

void simon_lights_set(enum_lights_state req_state) {
	serial_print_str("lt=");
	serial_print_int(req_state);
	lights_state = req_state;
	
	clear_pin(red_light_port, red_light_pin);
	clear_pin(blue_light_port, blue_light_pin);
	clear_pin(green_light_port, green_light_pin);
	clear_pin(yellow_light_port, yellow_light_pin);
	flashing_cycle = 0;
	flashing = 0;
	flashing_is_on = 0;
	flashing_cols = 0;
	
	switch(req_state) {
		case LT_ALL_OFF:
			break;
		case LT_ALL_ON:
			set_pin(red_light_port,		red_light_pin);
			set_pin(blue_light_port,	blue_light_pin);
			set_pin(green_light_port,	green_light_pin);
			set_pin(yellow_light_port,	yellow_light_pin);
			break;
		case LT_RED:
			serial_print_str("RED");
			set_pin(red_light_port, red_light_pin);
			break;
		case LT_BLUE:
			serial_print_str("BLU");
			set_pin(blue_light_port, blue_light_pin);
			break;
		case LT_GREEN:
			serial_print_str("GRN");
			set_pin(green_light_port, green_light_pin);
			break;
		case LT_YELLOW:
			serial_print_str("YEL");
			set_pin(yellow_light_port, yellow_light_pin);
			break;
		case LT_WHITE:
			serial_print_str("WHI");
			set_pin(white_light_port, white_light_pin);
			break;
		case LT_CYCLE:
			set_pin(red_light_port, red_light_pin);
			flashing_cycle = 1;
			break;
		case LT_RED_FLASH:
			set_pin(red_light_port, red_light_pin);
			flashing = 1;
			flashing_cols.flash_red = 1;
			flashing_is_on = 1;			
			break;
		case LT_BLUE_FLASH:
			set_pin(blue_light_port, blue_light_pin);
			flashing = 1;
			flashing_cols.flash_blue = 1;
			flashing_is_on = 1;
			break;
		case LT_GREEN_FLASH:
			set_pin(green_light_port, green_light_pin);
			flashing = 1;
			flashing_cols.flash_green = 1;
			flashing_is_on = 1;			
			break;
		case LT_YELLOW_FLASH:
			set_pin(yellow_light_port, yellow_light_pin);
			flashing = 1;
			flashing_cols.flash_yellow = 1;
			flashing_is_on = 1;			
			break;
		case LT_ALL_FLASH:
			flashing = 1;
			flashing_cols = 0xff;
			flashing_is_on = 1;
			
			set_pin(red_light_port, red_light_pin);
			set_pin(blue_light_port, blue_light_pin);
			set_pin(green_light_port, green_light_pin);
			set_pin(yellow_light_port, yellow_light_pin);
	}
	simon_lights_print();
}
			
void simon_lights_flash() {
	
	if (flashing_cycle) {
		if (flashing_cycle  == 1) {
			clear_pin(red_light_port, red_light_pin);
			set_pin(green_light_port, green_light_pin);
			flashing_cycle = 2;
		} else if (flashing_cycle == 2) {
			clear_pin(green_light_port, green_light_pin);
			set_pin(blue_light_port, blue_light_pin);
			flashing_cycle = 3;
		} else if (flashing_cycle == 3) {
			clear_pin(blue_light_port, blue_light_pin);
			set_pin(yellow_light_port, yellow_light_pin);
			flashing_cycle = 4;
		} else {
			clear_pin(yellow_light_port, yellow_light_pin);
			set_pin(red_light_port, red_light_pin);
			flashing_cycle = 1;
		}
	}	// cycling
	if (flashing) {
		if (flashing_is_on) {
			if (flashing_cols.flash_red) {
				clear_pin(red_light_port, red_light_pin);
			}
			if (flashing_cols.flash_green) {
				clear_pin(green_light_port, green_light_pin);
			}
			if (flashing_cols.flash_blue) {
				clear_pin(blue_light_port, blue_light_pin);
			}
			if (flashing_cols.flash_yellow) {
				clear_pin(yellow_light_port, yellow_light_pin);
			}
			flashing_is_on = 0;
		} else {	// flashing is off
			if (flashing_cols.flash_red) {
				set_pin(red_light_port, red_light_pin);
			}
			if (flashing_cols.flash_green) {
				set_pin(green_light_port, green_light_pin);
			}
			if (flashing_cols.flash_blue) {
				set_pin(blue_light_port, blue_light_pin);
			}
			if (flashing_cols.flash_yellow) {
				set_pin(yellow_light_port, yellow_light_pin);
			}
			flashing_is_on = 1;
		}	// flash cycle is off
	} // flashing			

}	

void simon_lights_setup_io() {
	clear_pin(red_light_port, red_light_pin);
	clear_pin(blue_light_port, blue_light_pin);
	clear_pin(green_light_port, green_light_pin);
	clear_pin(yellow_light_port, yellow_light_pin);
	clear_pin(white_light_port, white_light_pin);
	make_output(red_light_port, red_light_pin);
	make_output(blue_light_port, blue_light_pin);
	make_output(green_light_port, green_light_pin);
	make_output(yellow_light_port, yellow_light_pin);
	make_output(white_light_port, white_light_pin);
}